/**
 ********************************************************************
 * @file    camera_h264_hardware_decoder.h
 * @brief   This is the header file for "camera_h264_hardware_decoder.c", defining the structure and
 * (exported) function prototypes.
 *
 * @copyright (c) 2018 DJI. All rights reserved.
 *
 * All information contained herein is, and remains, the property of DJI.
 * The intellectual and technical concepts contained herein are proprietary
 * to DJI and may be covered by U.S. and foreign patents, patents in process,
 * and protected by trade secret or copyright law.  Dissemination of this
 * information, including but not limited to data and other proprietary
 * material(s) incorporated within the information, in any form, is strictly
 * prohibited without the express written consent of DJI.
 *
 * If you receive this source code without DJI’s authorization, you may not
 * further disseminate the information, and you must immediately remove the
 * source code and notify DJI of its removal. DJI reserves the right to pursue
 * legal actions against you for any loss(es) or damage(s) caused by your
 * failure to do so.
 *
 *********************************************************************
 */

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef CAMERA_H264_HARDWARE_DECODER_H
#define CAMERA_H264_HARDWARE_DECODER_H

/* Includes ------------------------------------------------------------------*/
#include <dji_liveview.hpp>
#include <dji_camera_manager.hpp>
#include "h264_idr_frame.hpp"
#include <thread>
#include <gst/gst.h>

using namespace DJI;
using namespace DJI::OSDK;

/* Exported constants --------------------------------------------------------*/
typedef struct {
	char format[16];
	int height;
	int width;

	uint8_t *imageData;
	uint32_t imageDataLen;
} T_Image;

typedef void (*ImageCallback)(const T_Image *image);

/* Exported types ------------------------------------------------------------*/
class CameraH264HardwareDecoder {
public:
	CameraH264HardwareDecoder(Vehicle *vehicle, PayloadIndexType payloadIndex);
	~CameraH264HardwareDecoder();
	ErrorCode::ErrorCodeType DecodeH264StreamData(const uint8_t *data, uint32_t dataLen);
	ErrorCode::ErrorCodeType RegisterRgbImageCallback(ImageCallback imageCallback);

private:
	Vehicle *m_vehicle;
	PayloadIndexType m_payloadIndex;

	bool m_exitUpdateRealIdrFrameTypeTask;
	T_OsdkTaskHandle m_updateRealIdrFrameTypeTaskHandle;

	bool m_exitGetNewFrameTask;
	bool m_IsGetNewFrameTaskLive;
	T_OsdkTaskHandle m_getNewFrameTaskHandle;

	IdrFrameType m_decoderUseIdrFrameType;
	IdrFrameType m_realIdrFrameType;
	ImageCallback m_imageCallback;

	T_OsdkMutexHandle m_gstMutexHandle;
	GstElement *m_gstPipeline, *m_gstAppSrc, *m_gstAppSink, *m_gstQueue, *m_gstH264Parse, *m_gstDecode;

	ErrorCode::ErrorCodeType CreateGstDecodePipeline();
	ErrorCode::ErrorCodeType DestoryGstDecodePipeline();
	ErrorCode::ErrorCodeType DecodeDataUseDecoder(const uint8_t *data, uint32_t dataLen);
	ErrorCode::ErrorCodeType RestartGstPipeline();

	friend void* UpdateRealIdrFrameTask(void *arg);
	friend void* GetNewFrameTask(void* arg);
};

#endif // CAMERA_H264_HARDWARE_DECODER_H
/************************ (C) COPYRIGHT DJI Innovations *******END OF FILE******/
